//
//  QtsHttpFileStation.h
//  QTSHTTPSDK
//
//  Created by Lawrence on 2013/11/24.
//  Copyright (c) 2013年 Lawrence. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "QtsHttpComm.h"

/** File station map to SDK version. */
typedef enum _QTS_HTTP_FILE_STATION_API_VERSION_
{
    /** Unknow version */
    QTS_HTTP_FILE_STATION_API_VERSION_UNKNOW = 0,
    /** File station API Version 1 */
    QTS_HTTP_FILE_STATION_API_V1   = 1,
    //QTS_FILESTATION_V2 = 2,
}QTS_HTTP_FILE_STATION_API_VERSION;

/** Compressed format for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_
{
    /** ZIP compressed format */
    QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_ZIP = 0,
    /** 7Z compressed format */
    QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE_7Z = 1,
}QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE;

/** Compressed level for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_
{
    /** Normal compressed level */
    QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_NORMAL = 0,
    /** The largest compressed level */
    QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_LARGE,
    /** The fastest compressed level */
    QTS_HTTP_FILE_STATION_COMPRESS_LEVEL_FAST,
}QTS_HTTP_FILE_STATION_COMPRESS_LEVEL;

/** Compressed encrypt type for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_
{
    /** AES256 encrypt */
    QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_AES256 = 0,
    /** ZIP encrypt */
    QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE_ZIPCRYPTO,
}QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE;

/** Compressed update mode for CompressFiles/CompressFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_
{
    /** Add and replace if file exist */
    QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_ADD_REPLACE = 0,
    /** Update and add if file exist */
    QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_UPDATE_ADD,
    /** Update exist file */
    QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_UPDATE_EXISTING,
    /** Synchronize update file */
    QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE_UPDATE_SYNCHRONIZE,
}QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE;

/** File overwrite mode for ExtractFiles/ExtractFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_OVERWRITE_
{
    /** Skip */
	QTS_HTTP_FILE_STATION_OVERWRITE_SKIP = 0,
	/** Overwrite */
    QTS_HTTP_FILE_STATION_OVERWRITE_OVERWRITE,
}QTS_HTTP_FILE_STATION_OVERWRITE;

/** Extract path mode for ExtractFiles/ExtractFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_
{
    /** No full path mode */
	QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_NONE = 0,
	/** Full path mode */
    QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE_FULL,
}QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE;

/** Extract mode for ExtractFiles/ExtractFilesAsync parameters */
typedef enum _QTS_HTTP_FILE_STATION_EXTRACT_MODE_
{
    /** Extract all files */
 	QTS_HTTP_FILE_STATION_EXTRACT_MODE_ALL = 0,
    /** Extract partial files */
 	QTS_HTTP_FILE_STATION_EXTRACT_MODE_PART,
}QTS_HTTP_FILE_STATION_EXTRACT_MODE;

/** Thumbnail size for photo. */
typedef enum _QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE_
{
    /** Large (800px thumbs) */
    QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE_LARGE = 0,
    /** Medium (400px thumb) */
    QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE_MEDIUM = 1,
    /** Small (100px thumb) */
    QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE_SMALL = 2
}QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE;
@interface QNQtsHttpFileEntry : NSObject
{
    BOOL      isDir;
    long long size;
    NSString* path;
    NSString* name;
}

@property (assign) BOOL      isDir; /** Is file or folder */
@property (assign) long long size;  /** File or folder size */
@property (strong) NSString* path;  /** File or folder path */
@property (strong) NSString* name;  /** File or folder name */
/*!
 * \brief
 * Constructor with parameters.
 *
 * \param
 * [IN] isDir : Is file or folder.
 * \param
 * [IN] size : File or folder size.
 * \param
 * [IN] path : File or folder path.
 * \param
 * [IN] name : File or folder name.
 *
 * \returns
 * QNQtsHttpFileEntry object instance.
 */
-(id) initWithPath:(NSString*)path name:(NSString*)name size:(long long)size isDir:(BOOL)isDir;
@end

@interface QNQtsHttpShareLinkInfo : NSObject
{
    NSString* shareFilePath;
    NSString* password;
    NSString* dateTime;
    BOOL      useSSL;
}

@property (strong) NSString* shareFilePath; /**Share file path include filename */
@property (strong) NSString* password;      /**Share with password. */
@property (strong) NSString* dateTime;      /**Share day and hour time. If dateTime = 0, share forever. */
@property (assign) BOOL      useSSL;        /** Share file with SSL */
/*!
 * \brief
 * Constructor with parameters.
 *
 * \param
 * [IN] filePath : Share file path include filename.
 * \param
 * [IN] password : Share file use password or not.
 * \param
 * [IN] dateTime : Share day and hour time.
 * \param
 * [IN] useSSL : Share file use SSL or not.
 *
 * \returns
 * QNQtsHttpShareLinkInfo object instance.
 */
-(id) initWithFilePath:(NSString*)filePath password:(NSString*)password datetime:(NSString*)dateTime useSSL:(BOOL)useSSL;
@end

@interface QNQtsHttpFileStatus : NSObject
{
    BOOL      isDir;
    BOOL      isCompressed;
    long long size;
    NSString* folderPermission;
    NSString* path;
    NSString* name;
    NSString* group;
    NSString* owner;
    NSString* mt;
}

@property (assign) BOOL      isDir;             /** Is file or folder */
@property (assign) BOOL      isCompressed;      /** Is compressed */
@property (assign) long long size;              /** File or folder size */
@property (strong) NSString* folderPermission;  /** File or folder permission */
@property (strong) NSString* path;              /** File or folder path */
@property (strong) NSString* name;              /** File or folder name */
@property (strong) NSString* group;             /** File or folder group */
@property (strong) NSString* owner;             /** File or folder owner */
@property (strong) NSString* mt;                /** File or folder modification time */
/*!
 * \brief
 * Constructor with parameters.
 *
 * \param
 * [IN] isDir : Is file or folder.
 * \param
 * [IN] isCompressed : Is compressed.
 * \param
 * [IN] size : File or folder size.
 * \param
 * [IN] folderPermission : File or folder permission.
 * \param
 * [IN] path : File or folder path.
 * \param
 * [IN] name : File or folder name.
 * \param
 * [IN] group : File or folder group.
 * \param
 * [IN] owner : File or folder owner.
 * \param
 * [IN] mt : File or folder modification time.
 *
 * \returns
 * QNQtsHttpFileStatus object instance.
 */
-(id) initWithPath:(NSString*)path name:(NSString*)name isDir:(BOOL)isDir
      isCompressed:(BOOL)isCompressed size:(long long)size folderPermission:(NSString*)folderPermission group:(NSString*)group owner:(NSString*)owner mt:(NSString*)mt;
@end

@interface QNQtsHttpShareLink : NSObject
{
    NSString* fileName;
    NSString* expireTime;
    NSString* url;
}

@property (strong) NSString* fileName;      /** Share link file name */
@property (strong) NSString* expireTime;    /** Share link expire time */
@property (strong) NSString* url;           /** Share link URL */
/*!
 * \brief
 * Constructor with parameters.
 *
 * \param
 * [IN] _filename : share file path and file name.
 * \param
 * [IN] _expireTime : share link expire date time.
 * \param
 * [IN] _url : share link access URL.
 *
 * \returns
 * QNQtsHttpShareLink object instance.
 */
-(id) initWithFileName:(NSString*)_filename expireTime:(NSString*)_expireTime url:(NSString*)_url;
@end

@interface QNQtsHttpExtractEntry : NSObject
{
    BOOL      isDir;
    long long size;
    long long compressSize;
    NSString* modifyTime;
    NSString* fileName;
}

@property (assign) BOOL      isDir;         /** Is file or folder */
@property (assign) long long size;          /** File or folder size */
@property (assign) long long compressSize;  /** File or folder compress size */
@property (strong) NSString* modifyTime;    /** File or folder modification time */
@property (strong) NSString* fileName;      /** File or folder full path */

@end

@interface QNQtsHttpNASInfo : NSObject
{
    NSString* myCloudNasName;   /** My cloud NAS name */
    NSString* nasExternalIP;    /** External IP of QNAP Turbo NAS */
    NSString* nasLocalIP;       /** Local IP of QNAP Turbo NAS */
    NSString* nasHosstIP;       /** Host IP of QNAP Turbo NAS */
}
/*!
 * \brief
 * Constructor with parameters.
 *
 * \param
 * [IN] strMyCloudNasName : My cloud NAS name.
 * \param
 * [IN] strNasExternalIP : External IP of QNAP Turbo NAS.
 * \param
 * [IN] strNasLocalIP : Local IP of QNAP Turbo NAS.
 * \param
 * [IN] strNasHostIP : Host IP of QNAP Turbo NAS.
 *
 * \returns
 * QNQtsHttpNASInfo object instance.
 */
-(id) initWithMyCloudNasName:(NSString*)strMyCloudNasNamw nasExternalIP:(NSString*)strNasExternalIP nasLocalIP:(NSString*)strNasLocalIP nasHostIP:(NSString*)strNasHostIP;
@end



@interface QNShareMember : NSObject

@property (strong,nonatomic) NSString *name;
@property (strong,nonatomic) NSNumber *share_member;
@property (strong,nonatomic) NSNumber *event_status;
@property (strong,nonatomic) NSNumber *event_id;

@end



/*!
 * \brief
 * Interface for Qts Http file station API
 *
 * 1. Support file station related API.
 *
 */
@protocol IQtsHttpFileStation <NSObject>
@optional

/*!
 * \brief
 * Get share folder list of QNAP Turbo NAS.
 *
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNQtsHttpFileEntry class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) getShareFolderListWithCommunicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get file list count below specify path.
 *
 * \param
 * [IN] path : Specify folder path.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store count with total tag.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 *
 * \remarks
 * File list count include files and folders.
 */
-(void) getFileListCountWithPath:(NSString*)path communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get file list below specify path.
 *
 * \param
 * [IN] path : Specify folder path.
 * \param
 * [IN] startIdx : Start index of get files.
 * \param
 * [IN] fileLimit : Number of get files.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNQtsHttpFileStatus class.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 *
 * \remarks
 * File list include files and folders.
 */
-(void) getFileListWithPath:(NSString*)path indexFrom:(int)startIdx fileLimit:(int)fileLimit communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Upload a file to QNAP Turbo NAS.
 *
 * \param
 * [IN] fromFilePath : Local path of upload file.
 * \param
 * [IN] toFilePath : Destination path in QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve upload progress rate and sending bytes.
 *
 * \returns
 * n/a.
 */
-(void) uploadFileWithPath:(NSString*)fromFilePath toPath:(NSString*)toFilePath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfUploading, unsigned long long bytesOfUploading))block;

/*!
 * \brief
 * Download a file to local.
 *
 * \param
 * [IN] fileEntry : Download file entry in QNAP Turbo NAS.
 * \param
 * [IN] toFilePath : Destination path in local.
 * \param
 * [IN] offset : Resume download from offset.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve download progress rate and received bytes.
 *
 * \returns
 * n/a.
 */
-(void) downloadFile:(QNQtsHttpFileEntry*)fileEntry toPath:(NSString*)toFilePath offset:(long)offset communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfDownloading, unsigned long long bytesOfDownloading))block;

/*!
 * \brief
 * Download a file by path to local.
 *
 * \param
 * [IN] path : Download file full path in QNAP Turbo NAS.
 * \param
 * [IN] toFilePath : Destination path in local.
 * \param
 * [IN] offset : Resume download from offset.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve download progress rate and received bytes.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code.
 */
-(void) downloadFileWithPath:(NSString*)path toPath:(NSString*)toFilePath offset:(long)offset communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfDownloading, unsigned long long bytesOfDownloadloading))block;

/*!
 * \brief
 * Create share link.
 *
 * \param
 * [IN] shareLinkInfo : Setting of create share link.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store QNQtsHttpShareLink class
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) createShareLink:(QNQtsHttpShareLinkInfo*)shareLinkInfo communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get share link list.
 *
 * \param
 * [IN] fileLimit : Number of share link list.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNQtsHttpShareLink class
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) getShareLinkListWithFileLimit:(int)fileLimit communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Delete share link.
 *
 * \param
 * [IN] shareLink : Share link on QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) deleteShareLink:(QNQtsHttpShareLink*)shareLink communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Create a folder.
 *
 * \param
 * [IN] parentFolderEntry : Create a folder below parentFolderEntry.
 * \param
 * [IN] folderName : Create folder name.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) createFolderWithParentFolder:(QNQtsHttpFileEntry*)parentFolderEntry name:(NSString*)folderName communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Copy a file/folder from source to destination.
 *
 * \param
 * [IN] fileEntry : Source entry in QNAP Turbo NAS.
 * \param
 * [IN] toPath : Destination path in QNAP Turbo NAS.
 * \param
 * [IN] mode : Skip / Overwrite.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve copy progress.
 *
 * \returns
 * n/a.
 */
-(void) copyFile:(QNQtsHttpFileEntry*)fileEntry toPath:(NSString*)toPath mode:(int)mode communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfCopy))block;

/*!
 * \brief
 * Move a file/folder from source to destination.
 *
 * \param
 * [IN] fileEntry : Source entry in QNAP Turbo NAS.
 * \param
 * [IN] toPath : Destination path in QNAP Turbo NAS.
 * \param
 * [IN] mode : Skip / Overwrite.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve move progress.
 *
 * \returns
 * n/a.
 */
-(void) moveFile:(QNQtsHttpFileEntry*)fileEntry toPath:(NSString*)toPath mode:(int)mode communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfMove))block;

/*!
 * \brief
 * Delete a file/folder in QNAP Turbo NAS.
 *
 * \param
 * [IN] fileEntry : Delete entry in QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) deleteFile:(QNQtsHttpFileEntry*)fileEntry communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Create a folder by path.
 *
 * \param
 * [IN] folderPath : Create a folder at this path.
 * \param
 * [IN] folderName : Create folder name.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) createFolderWithPath:(NSString*)folderPath name:(NSString*)folderName communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Copy a file/folder by path from source to destination.
 *
 * \param
 * [IN] fromPath : Source full path in QNAP Turbo NAS.
 * \param
 * [IN] toPath : Destination path in QNAP Turbo NAS.
 * \param
 * [IN] mode : Skip / Overwrite.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve copy progress.
 *
 * \returns
 * n/a.
 */
-(void) copyFileWithPath:(NSString*)fromPath toPath:(NSString*)toPath mode:(int)mode communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfCopy))block;

/*!
 * \brief
 * Move a file/folder by path from source to destination.
 *
 * \param
 * [IN] fromPath : Source full path in QNAP Turbo NAS.
 * \param
 * [IN] toPath : Destination path in QNAP Turbo NAS.
 * \param
 * [IN] mode : Skip / Overwrite.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve move progress.
 *
 * \returns
 * n/a.
 */
-(void) moveFileWithPath:(NSString*)fromPath toPath:(NSString*)toPath mode:(int)mode communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfMove))block;

/*!
 * \brief
 * Delete a file/folder by path in QNAP Turbo NAS.
 *
 * \param
 * [IN] filePath : Delete full path in QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) deleteFileWithPath:(NSString*)filePath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * File search.
 *
 * \param
 * [IN] folderPath :Search file path in QNAP Turbo NAS.
 * \param
 * [IN] keyword : Query keyword.
 * \param
 * [IN] fileLimit : Number of response file list.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNQtsHttpFileEntry objects.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.

 *
 * \returns
 * n/a.
 */
-(void) searchFileWithKeyword:(NSString*)keyword rootFolder:(NSString*)folderPath fileLimit:(int)fileLimit communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get file status.
 *
 * \param
 * [IN] filePath :File full path in QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store QNQtsHttpFileStatus objects.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) getFileStatusWithPath:(NSString*)filePath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Create chunked upload ID.
 *
 * \param
 * [IN] uploadTempPath : Where to put temp file in QTS Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store upload_id with upload_id tag.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) createChunkedUploadIdWithPath:(NSString*)uploadTempPath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Chunked upload a file to QNAP Turbo NAS.
 *
 * \param
 * [IN] uploadFilePath : Local path of chunked upload file.
 * \param
 * [IN] toFilePath : Destination path in QNAP Turbo NAS.
 * \param
 * [IN] renameOriDestFileName : Rename original destination file to strRenameOriDestFileName, if file already exist.
 * \param
 * [IN] uploadID : Chunked upload ID. Get from CreateChunkedUploadID.
 * \param
 * [IN] uploadTempPath : Where to put temp file in QTS Turbo NAS. The same as CreateChunkedUploadID.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 * \param
 * [BLOCK] block : block to retrieve upload progress.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code.
 */
-(void) chunkedUploadFileWithPath:(NSString*)uploadFilePath toPath:(NSString*)toFilePath fileName:(NSString*)renameOriDestFileName uploadID:(NSString*)uploadID uploadTempPath:(NSString*)uploadTempPath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfUploading, unsigned long long bytesOfUploading))block;

/*!
 * \brief
 * Delete chunked upload file by upload ID.
 *
 * \param
 * [IN] uploadID : Chunked upload ID. Get from CreateChunkedUploadID.
 * \param
 * [IN] uploadTempPath : Where to put temp file in QTS Turbo NAS. The same as CreateChunkedUploadID.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) deleteChunkedUploadFileWithId:(NSString*)uploadID uploadTempPath:(NSString*)uploadTempPath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Rename a file/folder in QNAP Turbo NAS.
 *
 * \param
 * [IN] filePath : Rename file path.
 * \param
 * [IN] newName : New folder/file name.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) renameFileWithPath:(NSString*)filePath fileName:(NSString*)newName communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Set folder/file modification time.
 *
 * \param
 * [IN] filePath : Update file path.
 * \param
 * [IN] modifyTime : Epoch time (seconds since 1970-01-01 00:00:00 UTC).
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) updateFileModifyDateWithFilePath:(NSString*)filePath modifyTime:(long)modifyTime communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Compress files.
 *
 * \param
 * [IN] compressName :Compressed name.
 * \param
 * [IN] compressFilePath : Compressd file path.
 * \param
 * [IN] totalFileNum : The amount of compression files number.
 * \param
 * [IN] fileList : The compressed file name list.
 * \param
 * [IN] compressFormat : Compressed format.
 * \param
 * [IN] password : Compressed password (can be null).
 * \param
 * [IN] compressLevel : Compressed level.
 * \param
 * [IN] encryptType : Compressed encrypt type.
 * \param
 * [IN] updateMode : Compressed update mode.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code.
 */
-(void) compressFilesName:(NSString*)compressName path:(NSString*)compressFilePath numberOfFiles:(int)totalFileNum fileList:(NSMutableArray*)fileList compressType:(QTS_HTTP_FILE_STATION_COMPRESS_FORMAT_TYPE)compressFormat password:(NSString*)password compressLevel:(QTS_HTTP_FILE_STATION_COMPRESS_LEVEL)compressLevel compressEncryrtType:(QTS_HTTP_FILE_STATION_COMPRESS_ENCRYPT_TYPE)encryptType compressUpdateMode:(QTS_HTTP_FILE_STATION_COMPRESS_UPDATE_MODE)updateMode communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * List the contents of an extracted file.
 *
 * \param
 * [IN] filePath :Extracted file full path in QTS Turbo NAS.
 * \param
 * [IN] startIdx : Response files start index.
 * \param
 * [IN] fileLimit : Number of response files.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNQtsHttpExtractEntry objects.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) ListContentsOfExtractedFilePath:(NSString*)filePath start:(int)startIdx limit:(int)fileLimit communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Extract files.
 *
 * \param
 * [IN] extractedFile : Extracted file object in QTS Turbo NAS.
 * \param
 * [IN] toPath : Path of the extracted files.
 * \param
 * [IN] password : Extraction password (can be null).
 * \param
 * [IN] extractMode : Extraction mode.
 * \param
 * [IN] partTotalFileCount : The total number of extracted files, if extractMode = QTS_EXTRACT_MODE_PART.
 * \param
 * [IN] partFileList : Name of the file to be extracted (can be more than one), if extractMode = QTS_EXTRACT_MODE_PART.
 * \param
 * [IN] overWrite : Skip / Overwrite.
 * \param
 * [IN] extractPathMode : Extracted path mode.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) ExtractFiles:(QNQtsHttpFileEntry*)extractedFile toPath:(NSString*)toPath password:(NSString*)password exractMode:(QTS_HTTP_FILE_STATION_EXTRACT_MODE)extractMode partTotalFileCount:(int)partTotalFileCount partFileList:(NSMutableArray*)partFileList overwrite:(QTS_HTTP_FILE_STATION_OVERWRITE)overWrite extractPathMode:(QTS_HTTP_FILE_STATION_EXTRACT_PATH_MODE)extractPathMode communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get hostname and external IP address of the QNAP Turbo NAS.
 *
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store list of QNQtsHttpNASInfo objects
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code.
 */
-(void) getDomainIPListWithCommunicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Add video to transcode queue.
 *
 * \param
 * [IN] filePath : Add video file path in QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) addVideoTranscodeWithFilePath:(NSString*)filePath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Delete video transcode file.
 *
 * \param
 * [IN] filePath : Delete video file path in QNAP Turbo NAS.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * QTS_HTTP_API_RESULTS : API success or fail error code.
 */
-(void) deleteVideoTranscodeWithFilePath:(NSString*)filePath communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

/*!
 * \brief
 * Get file URL.
 *
 * \param
 * [IN] fileFullPath : File full path.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store file URL.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) getFileURLWithFilePath:(NSString*)fileFullPath cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;
/*!
 * \brief
 * Get image thumbnail of file.
 *
 * \param
 * [IN] fileFullPath : File full path.
 * \param
 * [IN] isAsync : Execute API by Synchronized or Asynchronized Mode.
 * \param
 * [IN] isCancel : Cancel API.
 * \param
 * [BLOCK] success : block to retrieve QTS_RESPONSE_CTX class when API was executed successfully.
 *                   QTS_RESPONSE_CTX's response will store image thumv URL.
 * \param
 * [BLOCK] fail : block to retrieve error code when API was executed failurely.
 *
 * \returns
 * n/a.
 */
-(void) getImgThumbURLWithFilePath:(NSString*)fileFullPath cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

-(void) getImgThumbURLWithFilePath:(NSString*)fileFullPath thumbnailSize:(QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE)thumbnailSize cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

-(void) getImgThumbURLWithFilePath:(NSString*)fileFullPath isSSL:(BOOL)isSSL thumbnailSize:(QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE)thumbnailSize cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

-(void) getImgThumbURLWithFile:(QNQtsHttpFileEntry*)entry thumbnailSize:(QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE)thumbnailSize cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

-(void) getImgThumbURLWithFile:(QNQtsHttpFileEntry*)entry isSSL:(BOOL)isSSL thumbnailSize:(QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE)thumbnailSize cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail;

-(void) downloadImgThumbWithFile:(QNQtsHttpFileEntry*)entry toPath:(NSString*)toFilePath thumbnailSize:(QTS_HTTP_FILE_STATION_THUMBNAIL_SIZE)thumbnailSize communicationMode:(BOOL)isAsync cancel:(int*)isCancel success:(void(^)(QTS_RESPONSE_CTX* response))success fail:(void(^)(QTS_HTTP_API_RESULTS error))fail progress:(void(^)(int rateOfDownloading, unsigned long long bytesOfDownloading))block;

@end
@interface QNQtsHttpFileStation : NSObject <IQtsHttpFileStation>
@end
