//
//  AppDelegate.m
//  OSX_QtsHttp_Sample
//
//  Created by Lawrence on 2013/11/27.
//  Copyright (c) 2013年 Lawrence. All rights reserved.
//

#import "AppDelegate.h"

#define FOLDER_TYPE @"FOLDER"
#define FILE_TYPE   @"FILE"

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    currentNasLocationPath = @"\\";
    [tblNasSystem setDelegate:self];
    [tblNasSystem setDataSource:self];
    [tblNasSystem setTarget:self];
    [tblNasSystem setDoubleAction:@selector(goNasNext)];
    
    [tblLocalSystem setDelegate:self];
    [tblLocalSystem setDataSource:self];
    [tblLocalSystem setTarget:self];
    [tblLocalSystem setDoubleAction:@selector(goLocalNext)];
    
    tableLocalDateSource = [[NSMutableArray alloc] init];
    tableNasDateSource = [[NSMutableArray alloc] init];
    
    NSArray *documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    rootPathOfLocalFileSystem = [[documentPaths objectAtIndex:0] stringByDeletingLastPathComponent];
    currentLocalLocationPath = rootPathOfLocalFileSystem;
    [self getLocalFileList:currentLocalLocationPath];
    
    isAbort = 0;
}

//// File List
-(void)getFolderList
{
    isAbort = 0;
    [[server openFileStation] getShareFolderListWithCommunicationMode:YES cancel:&isAbort
    success:^(QTS_RESPONSE_CTX *response) {
        currentNasLocationPath = @"/";
        tableNasDateSource = (NSMutableArray*)response.response;
        [tblNasSystem reloadData];
    } fail:^(QTS_HTTP_API_RESULTS error) {
        NSLog(@"GET SHARE FOLDER LIST FAIL.");
    }];
}

-(void)getFileList:(NSString*)path
{
    @synchronized(tableNasDateSource){
        isAbort = 0;
        [[server openFileStation] getFileListCountWithPath:path communicationMode:YES cancel:&isAbort
        success:^(QTS_RESPONSE_CTX *response) {
            int total = [[[response.response objectAtIndex:0] objectForKey:@"count"] intValue];
            [tableNasDateSource removeAllObjects];
            QNQtsHttpFileStatus* entryOfLastPath = [[QNQtsHttpFileStatus alloc] initWithPath:[path stringByDeletingLastPathComponent] name:@".." isDir:YES isCompressed:NO size:0 folderPermission:@"" group:@"" owner:@"" mt:@""];
            [tableNasDateSource addObject:entryOfLastPath];
            
            for ( int idx = 0 ; idx < total ;  idx = (idx+1000) )
            {
                [[server openFileStation] getFileListWithPath:path indexFrom:idx fileLimit:1000 communicationMode:YES cancel:&isAbort
                success:^(QTS_RESPONSE_CTX *response) {
                    for ( QNQtsHttpFileStatus* item in response.response ) {
                        [tableNasDateSource addObject:item];
                        currentNasLocationPath = path;
                        [tblNasSystem reloadData];
                    }
                } fail:^(QTS_HTTP_API_RESULTS error) {
                    NSLog(@"GET FILE LIST FAIL.");
                }];
            }
        } fail:^(QTS_HTTP_API_RESULTS error) {
            NSLog(@"GET FILE LIST COUNT FAIL.");
        }];
    }
}

-(void)getLocalFileList:(NSString*)path
{
    @synchronized(tableLocalDateSource)
    {
        NSFileManager *fileManager = [NSFileManager defaultManager];
        NSError *error = nil;
        //fileList便是包含有该文件夹下所有文件的文件名及文件夹名的数组
        NSArray* fileList = [fileManager contentsOfDirectoryAtPath:path error:&error];
        [tableLocalDateSource removeAllObjects];
        
        if ( ![rootPathOfLocalFileSystem isEqualToString:path] ) {
            QNQtsHttpFileStatus* item = [[QNQtsHttpFileStatus alloc] initWithPath:[path stringByDeletingLastPathComponent] name:@".." isDir:YES isCompressed:NO size:0 folderPermission:@"" group:@"" owner:@"" mt:@""];
            [tableLocalDateSource addObject:item];
        }
        
        for ( NSString *file in fileList ) {
            if ( [file hasPrefix:@"."] ) continue;
            NSString *path = [currentLocalLocationPath stringByAppendingPathComponent:file];
            BOOL isDir = NO;
            [fileManager fileExistsAtPath:path isDirectory:(&isDir)];
            if ( isDir ) {
                QNQtsHttpFileStatus* item = [[QNQtsHttpFileStatus alloc] initWithPath:path name:file isDir:YES isCompressed:NO size:0 folderPermission:@"" group:@"" owner:@"" mt:@""];
                [tableLocalDateSource addObject:item];
            }
            else {
                QNQtsHttpFileStatus* item = [[QNQtsHttpFileStatus alloc] initWithPath:path name:file isDir:NO isCompressed:NO size:0 folderPermission:@"" group:@"" owner:@"" mt:@""];
                [tableLocalDateSource addObject:item];
            }
        }
        
        [tblLocalSystem reloadData];
    }
}

//// NSTableView Double Click
-(void)goNasNext
{
    int iSelectedRow = (int)[tblNasSystem selectedRow];
    QNQtsHttpFileStatus* item = [tableNasDateSource objectAtIndex:iSelectedRow];
    
    if ( item.isDir ) {
        if ( [currentNasLocationPath isEqualToString:@"/"] ) {
            [self getFileList:[NSString stringWithFormat:@"/%@",item.name]];
        }
        else if ( iSelectedRow > 0 ) {
            [self getFileList:[NSString stringWithFormat:@"%@/%@",item.path,item.name]];
        }
        else {
            if ( [item.path isEqualToString:@"/"] ) {
                [self getFolderList];
            }
            else {
                [self getFileList:item.path];
            }
        }
    }
}

-(void)goLocalNext
{
    int iSelectedRow = (int)[tblLocalSystem selectedRow];
    QNQtsHttpFileStatus* item = [tableLocalDateSource objectAtIndex:iSelectedRow];
    if ( item.isDir ) {
        currentLocalLocationPath = item.path;
        [self getLocalFileList:item.path];
    }
}

//// NSButton Action
-(IBAction)access:(id)sender
{
    if ( [btnAccess.title isEqualToString:@"LOGIN"] ) {
        serverInfo = [[QNQtsHttpServerInfo alloc] initWithHostName:tfIPAddress.stringValue username:tfUserName.stringValue password:tfPassword.stringValue isSecureConnection:[_sslEnable state] computerName:@""];
        
        server = [[QNQtsHttpServer alloc] initWithServerInfo:serverInfo];
        if ([_sslEnable state] == NSOnState)
           [server setFileStationSSLPortNum:[tfPort.stringValue longLongValue]];
        else
            [server setFileStationPortNum:[tfPort.stringValue longLongValue]];
        isAbort = 0;
        [server login:QTS_HTTP_STATION_TYPE_FILE_STATION communicationMode:YES cancel:&isAbort
        success:^(QTS_RESPONSE_CTX *response) {
            if ( response.result == QTS_HTTP_APIS_SUCCESS ) {
                btnAccess.title = @"LOGOUT";
                [self getFolderList];
            }
        } fail:^(QTS_HTTP_API_RESULTS error) {
            NSLog(@"USER LOGIN FAIL.");
        }];
    }
    else if ( [btnAccess.title isEqualToString:@"LOGOUT"] ) {
        isAbort = 0;
        [server closeFileStation:[server openFileStation] communicationMode:YES cancel:&isAbort success:^(QTS_RESPONSE_CTX *response) {
            if ( response.result == QTS_HTTP_APIS_SUCCESS ) {
                btnAccess.title = @"LOGIN";
                lblProgress.stringValue = @"";
                server = nil;
                serverInfo = nil;
                [tableNasDateSource removeAllObjects];
                [tblNasSystem reloadData];
            }
        } fail:^(QTS_HTTP_API_RESULTS error) {
            NSLog(@"USER LOGOUT FAIL.");
        }];
    }
}

-(IBAction)upload:(id)sender
{
    int iSelectedRow = (int)[tblLocalSystem selectedRow];
    QNQtsHttpFileStatus* item = [tableLocalDateSource objectAtIndex:iSelectedRow];
    isAbort = 0;
    [[server openFileStation] uploadFileWithPath:item.path toPath:currentNasLocationPath communicationMode:YES cancel:&isAbort
    success:^(QTS_RESPONSE_CTX *response) {
        if ( response.result == QTS_HTTP_APIS_SUCCESS ) {
            [self getFileList:currentNasLocationPath];
            NSLog(@"UPLOAD FILE DONE.");
        }
    } fail:^(QTS_HTTP_API_RESULTS error) {
        NSLog(@"UPLOAD FILE FAIL.(ErrorCode=%d)",error);
    } progress:^(int rateOfUploading, unsigned long long bytesOfUploading) {
        lblProgress.stringValue = [NSString stringWithFormat:@"%d",rateOfUploading];
    }];
}

-(IBAction)download:(id)sender
{
    int iSelectedRow = (int)[tblNasSystem selectedRow];
    QNQtsHttpFileStatus* item = [tableNasDateSource objectAtIndex:iSelectedRow];
    QNQtsHttpFileEntry* entry = [[QNQtsHttpFileEntry alloc] initWithPath:item.path name:item.name size:item.size isDir:item.isDir];
    isAbort = 0;
    [[server openFileStation] downloadFile:entry toPath:currentLocalLocationPath offset:0 communicationMode:YES cancel:&isAbort
    success:^(QTS_RESPONSE_CTX *response) {
        if ( response.result == QTS_HTTP_APIS_SUCCESS ) {
            [self getLocalFileList:currentLocalLocationPath];
            NSLog(@"DOWNLOAD FILE DONE.");
        }
    } fail:^(QTS_HTTP_API_RESULTS error) {
        NSLog(@"DOWNLOAD FILE FAIL.(ErrorCode=%d)",error);
    } progress:^(int rateOfDownloading, unsigned long long bytesOfDownloading) {
        lblProgress.stringValue = [NSString stringWithFormat:@"%d",rateOfDownloading];
    }];
}

-(IBAction)stop:(id)sender
{
    isAbort = 1;
}

//// NSTableView Delegate
- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
    if ( tableView == tblNasSystem ) {
        return [tableNasDateSource count];
    }
    else if ( tableView == tblLocalSystem ) {
        return [tableLocalDateSource count];
    }
    return 0;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    if ( tableView == tblNasSystem )
    {
        if ( [[tableNasDateSource objectAtIndex:row] isKindOfClass:[QNQtsHttpFileStatus class]] ) {
            QNQtsHttpFileStatus* item = [tableNasDateSource objectAtIndex:row];
            if ( [tableColumn.identifier isEqualToString:@"FileType"] ) {
                return item.isDir?FOLDER_TYPE:FILE_TYPE;
            }
            else if ( [tableColumn.identifier isEqualToString:@"FileName"] ) {
                return item.name;
            }
        }
        else if ( [[tableNasDateSource objectAtIndex:row] isKindOfClass:[QNQtsHttpFileEntry class]] ) {
            QNQtsHttpFileEntry* item = [tableNasDateSource objectAtIndex:row];
            if ( [tableColumn.identifier isEqualToString:@"FileType"] ) {
                return item.isDir?FOLDER_TYPE:FILE_TYPE;
            }
            else if ( [tableColumn.identifier isEqualToString:@"FileName"] ) {
                return item.name;
            }
        }
        
    }
    else if ( tableView == tblLocalSystem )
    {
        if ( [[tableLocalDateSource objectAtIndex:row] isKindOfClass:[QNQtsHttpFileStatus class]] ) {
            QNQtsHttpFileStatus* item = [tableLocalDateSource objectAtIndex:row];
            if ( [tableColumn.identifier isEqualToString:@"FileType"] ) {
                return item.isDir?FOLDER_TYPE:FILE_TYPE;
            }
            else if ( [tableColumn.identifier isEqualToString:@"FileName"] ) {
                return item.name;
            }
        }
    }
    return nil;
}


@end
